// generated by Fast Light User Interface Designer (fluid) version 1.0011

#ifndef panels_H
#define panels_H
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
extern Fl_Window *disk_chooser;
#include <FL/Fl_Button.H>
extern void reload_cb(Fl_Button*, void*);
extern void copyright_cb(Fl_Button*, void*);
#include <FL/Fl_Browser.H>
extern void disk_browser_cb(Fl_Browser*, void*);
extern Fl_Browser *disk_browser;
#include <FL/Fl_Input.H>
extern void disk_input_cb(Fl_Input*, void*);
extern Fl_Input *disk_input;
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Check_Button.H>
extern Fl_Check_Button *all_files_button;
Fl_Window* make_diskchooser();
extern Fl_Window *wait_window;
#include <FL/Fl_Slider.H>
extern Fl_Slider *wait_slider;
extern void cancel_cb(Fl_Button*, void*);
Fl_Window* make_wait_window();
extern Fl_Window *print_panel;
#include <FL/Fl_Box.H>
#include <FL/Fl_Light_Button.H>
extern Fl_Input *print_command_input;
extern Fl_Light_Button *print_file_button;
extern Fl_Input *print_file_input;
#include <FL/Fl_Group.H>
extern Fl_Check_Button *print_portrait_button;
extern Fl_Check_Button *fill_page_button;
#include <FL/Fl_Return_Button.H>
extern Fl_Return_Button *print_ok_button;
Fl_Window* make_print_panel();
#endif
